package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.web.core.util.WebFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.supplierprocurementplan.BpmOASupplierProcurementPlanPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.supplierprocurementplan.BpmOASupplierProcurementPlanSaveReqVO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOASupplierProcurementPlanConvert;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOASupplierProcurementPlanItemConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOASupplierProcurementPlanDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOASupplierProcurementPlanItemDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOASupplierProcurementPlanItemMapper;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOASupplierProcurementPlanMapper;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 供应商采购计划 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class BpmOASupplierProcurementPlanServiceImpl extends BpmOABaseService implements BpmOASupplierProcurementPlanService {

    @Resource
    private BpmOASupplierProcurementPlanMapper oaSupplierProcurementPlanMapper;
    @Resource
    private BpmOASupplierProcurementPlanItemMapper oaSupplierProcurementPlanItemMapper;
    @Resource
    private BpmProcessInstanceApi processInstanceApi;
    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;
//    @Resource
//    private BpmProcessInstanceService processInstanceService;

    /**
     * OA 供应商资产采购申请 KEY
     */
    public static final String PROCESS_KEY = "oa_supplier_procurement_plan_2";

    @Override
    public Long createOaSupplierProcurementPlan(BpmOASupplierProcurementPlanSaveReqVO createReqVO) {
        // 插入
        BpmOASupplierProcurementPlanDO oaSupplierProcurementPlan = BeanUtils.toBean(createReqVO, BpmOASupplierProcurementPlanDO.class);
        oaSupplierProcurementPlanMapper.insert(oaSupplierProcurementPlan);
        // 返回
        Long userId = WebFrameworkUtils.getLoginUserId();
        createReqVO.setUserId(userId);
        // 采购支付对象转换
        BpmOASupplierProcurementPlanDO oaSupplierProcurementPlanDO = BpmOASupplierProcurementPlanConvert.INSTANCE.convert(createReqVO);
        oaSupplierProcurementPlanMapper.insert(oaSupplierProcurementPlanDO);

        // 子列表转换
        List<BpmOASupplierProcurementPlanItemDO> items = BpmOASupplierProcurementPlanItemConvert.INSTANCE.convert(createReqVO.getItems());
        for (BpmOASupplierProcurementPlanItemDO item : items) {
            item.setProcurementPlanId(oaSupplierProcurementPlanDO.getId());
            item.setUserId(userId);
        }
        oaSupplierProcurementPlanItemMapper.insertBatch(items);
        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(oaSupplierProcurementPlan.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 采购单中
        oaSupplierProcurementPlanMapper.updateById(new BpmOASupplierProcurementPlanDO().setId(oaSupplierProcurementPlan.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {
            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> purchaseFileItems = createReqVO.getPurchaseFileItems();
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (purchaseFileItems != null && !purchaseFileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId, purchaseFileItems);
        }
//        List<UploadUserFile> theArrivalFileItems = createReqVO.getTheArrivalFileItems();
//        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
//        if (theArrivalFileItems != null && !theArrivalFileItems.isEmpty()) {
//            uploadBpmFileProcessInstanceId(processInstanceId, theArrivalFileItems);
//        }
        // 返回
        return oaSupplierProcurementPlanDO.getId();
    }

    @Override
    public void updateOaSupplierProcurementPlan(BpmOASupplierProcurementPlanSaveReqVO updateReqVO) {
        // 更新
        BpmOASupplierProcurementPlanDO updateObj = BeanUtils.toBean(updateReqVO, BpmOASupplierProcurementPlanDO.class);
        oaSupplierProcurementPlanMapper.updateById(updateObj);
    }

    @Override
    public void deleteOaSupplierProcurementPlan(Long id) {
        // 删除
        oaSupplierProcurementPlanMapper.deleteById(id);
    }


    @Override
    public BpmOASupplierProcurementPlanDO getOaSupplierProcurementPlan(Long id) {
        return oaSupplierProcurementPlanMapper.selectById(id);
    }

    @Override
    public PageResult<BpmOASupplierProcurementPlanDO> getOaSupplierProcurementPlanPage(BpmOASupplierProcurementPlanPageReqVO pageReqVO) {
        return oaSupplierProcurementPlanMapper.selectPage(pageReqVO);
    }

    @Override
    public void updateSupplierProcurementPlanResult(String processInstanceId, Long id, Integer result) {
        oaSupplierProcurementPlanMapper.updateById(new BpmOASupplierProcurementPlanDO().setId(id).setResult(result).setProcessInstanceId(processInstanceId));
    }

    @Override
    public void batchEditSettlementFlag(List<Long> ids, Integer settlementFlag) {
        oaSupplierProcurementPlanMapper.update(new BpmOASupplierProcurementPlanDO().setSettlementFlag(settlementFlag),
                new LambdaQueryWrapper<BpmOASupplierProcurementPlanDO>()
                        .in(BpmOASupplierProcurementPlanDO::getId, ids));
    }

}
