package cn.iocoder.yudao.module.system.dal.mysql.worklog;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.datapermission.core.annotation.DataPermission;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.MPJLambdaWrapperX;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstancePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceRespVO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.PostDO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.UserPostDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogInstanceDO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogReadDo;
import cn.iocoder.yudao.module.system.service.worklog.dto.LogReadUserRespDTO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 日志实例的拓展 Mapper
 *
 * @author 符溶馨
 */
@Mapper
public interface LogInstanceMapper extends BaseMapperX<LogInstanceDO> {


    default PageResult<LogInstanceDO> getUpLog(Long userId, Long formId) {

        return selectPage(new PageParam().setPageNo(1).setPageSize(1), new LambdaQueryWrapperX<LogInstanceDO>()
                .eq(LogInstanceDO::getStartUserId, userId)
                .eqIfPresent(LogInstanceDO::getFormId, formId)
                .orderByDesc(LogInstanceDO::getCreateTime));
    }

    default Long getLogByFormId(Long formId, Long userId, List<String> dateList) {

        return selectCount(new LambdaQueryWrapperX<LogInstanceDO>()
                .eq(LogInstanceDO::getFormId, formId)
                .eq(LogInstanceDO::getStartUserId, userId)
                .in(LogInstanceDO::getTime, dateList));
    }

    default IPage<LogInstanceRespVO> selectPageResult(@Param("page") IPage<LogInstanceRespVO> page,
                                                      @Param("reqVO") LogInstancePageReqVO reqVO,
                                                      @Param("userId") Long userId,
                                                      @Param("pagingType") Integer pagingType,
                                                      @Param("userIds") List<Long> userIds) {

        MPJLambdaWrapperX<LogInstanceDO> queryWrapper = new MPJLambdaWrapperX<>();
        queryWrapper.selectAll(LogInstanceDO.class);
        queryWrapper.selectAs("e.read_status", LogInstanceRespVO::getReadStatus);
        queryWrapper.selectAs("COALESCE(c.readCount, 0)", LogInstanceRespVO::getReadCount);
        queryWrapper.selectAs("COALESCE(d.unreadCount, 0)", LogInstanceRespVO::getUnreadCount);
        queryWrapper.selectAs("COALESCE(b.comment, 0)", LogInstanceRespVO::getComment);
        queryWrapper.innerJoin(UserPostDO.class, "userPost", UserPostDO::getUserId, LogInstanceDO::getStartUserId);
        queryWrapper.innerJoin(DeptDO.class, "dept", DeptDO::getId, LogInstanceDO::getDeptId);
        queryWrapper.innerJoin(PostDO.class, "post", PostDO::getId, UserPostDO::getPostId);
        queryWrapper.leftJoin(LogReadDo.class, "e", on -> on
                .eq(LogReadDo::getLogInstanceId, LogInstanceDO::getId)
                .eq(LogReadDo::getReadUserId, userId)
                .eq(LogReadDo::getDeleted, 0));
        queryWrapper.leftJoin("(SELECT log_instance_id, COUNT(log_instance_id) AS readCount FROM work_log_read where read_status = 1 GROUP BY log_instance_id) c on t.id = c.log_instance_id");
        queryWrapper.leftJoin("(SELECT log_instance_id, COUNT(log_instance_id) AS unReadCount FROM work_log_read where read_status = 0 GROUP BY log_instance_id) AS d ON t.id = d.log_instance_id");
        queryWrapper.leftJoin("(SELECT work_log_id, COUNT(work_log_id) AS comment FROM work_log_comment GROUP BY work_log_id) AS b ON t.id = b.work_log_id");
        queryWrapper.eqIfPresent(LogInstanceDO::getDeptId, reqVO.getDeptId());
        queryWrapper.eqIfPresent(LogInstanceDO::getFormId, reqVO.getFormId());
        queryWrapper.eqIfPresent(LogInstanceDO::getStartUserId, reqVO.getStartUserId());
        queryWrapper.betweenIfPresent(LogInstanceDO::getTime, reqVO.getCreateTime());
        queryWrapper.inIfPresent(LogInstanceDO::getStartUserId, userIds);

        if (reqVO.getReadStatus() != null) {
            queryWrapper.eq("e.read_status", reqVO.getReadStatus());
        }
        if (reqVO.getIsProduce() != null && reqVO.getIsProduce() == 1) {
            queryWrapper.like(DeptDO::getFlag, "166");
        }
        if (reqVO.getIsProduce() != null && reqVO.getIsProduce() == 2) {
            queryWrapper.notLike(DeptDO::getFlag, "166");
        }
        if (pagingType == 0) {
            queryWrapper.ne(LogInstanceDO::getStartUserId, userId);
        }
        if (pagingType == 1) {
            queryWrapper.eq(LogInstanceDO::getStartUserId, userId);
        }
        queryWrapper.groupBy(LogInstanceDO::getId);
        queryWrapper.orderByDesc(LogInstanceDO::getTime);
        queryWrapper.orderByAsc(PostDO::getSort);
        queryWrapper.orderByDesc(LogInstanceDO::getCreateTime);

        return selectJoinPage(page, LogInstanceRespVO.class, queryWrapper);
    }

    default PageResult<LogInstanceDO> selectPage(LogInstancePageReqVO reqVO, Long userId,
                                                  Integer pagingType, List<Long> userIds) {

        LambdaQueryWrapperX<LogInstanceDO> queryWrapper = new LambdaQueryWrapperX<LogInstanceDO>()
                .eqIfPresent(LogInstanceDO::getFormId, reqVO.getFormId())
                .eqIfPresent(LogInstanceDO::getDeptId, reqVO.getDeptId())
                .eqIfPresent(LogInstanceDO::getStartUserId, reqVO.getStartUserId())
                .betweenIfPresent(LogInstanceDO::getTime, reqVO.getCreateTime());

        if (pagingType == 0) {
            queryWrapper.ne(LogInstanceDO::getStartUserId, userId);
        } else if (pagingType == 1) {
            queryWrapper.eq(LogInstanceDO::getStartUserId, userId);
        }

        if (reqVO.getIsBoss() != null && reqVO.getIsProduce() == null) {
            queryWrapper.inIfPresent(LogInstanceDO::getStartUserId, userIds);
        }

        return selectPage(reqVO, queryWrapper.orderByDesc(LogInstanceDO::getTime));

    }

    @DataPermission(enable = false)
    List<LogReadUserRespDTO> selectRaedUser(@Param("userId")Long userId, @Param("deptId")Long deptId);

    /**
     * 获取上一篇下一篇
     *
     * @param dto
     * @param id
     * @param userId
     * @param type
     * @return
     */
    LogInstanceRespVO getNextOrUp(@Param("reqVO") LogInstancePageReqVO dto,
                                  @Param("id") Long id, @Param("userId") Long userId,
                                  @Param("type") Integer type,
                                  @Param("pagingType") Integer pagingType,
                                  @Param("userIds") List<Long> userIds);
}