package cn.iocoder.yudao.module.system.job.birthday;

import cn.hutool.json.JSONObject;
import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.framework.tenant.core.job.TenantJob;
import cn.iocoder.yudao.module.system.api.subscribe.SubscribeMessageSendApi;
import cn.iocoder.yudao.module.system.convert.notice.NoticeConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.dal.mysql.user.AdminUserMapper;
import cn.iocoder.yudao.module.system.service.permission.PermissionService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.context.XxlJobContext;
import com.xxl.job.core.handler.annotation.XxlJob;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Set;

import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;

@Component
@Slf4j
@EnableScheduling
public class BirthdayJob {

    @Resource
    private AdminUserMapper adminUserMapper;

    @Resource
    private AdminUserService userService;

    @Resource
    private PermissionService permissionService;

    @Resource
    private SubscribeMessageSendApi subscribeMessageSendApi;

    /**
     * 生日提醒
     *
     * @return
     */
    @XxlJob("birthdayJob")
    @TenantJob // --- ⚠️ 这个注解 会将租户列表拉出来 完了后逐个租户执行 定时任务需要注意
    public ReturnT<String> execute() {

        // 生日提醒
        birthdayRemind();

        // 返回执行成功
        return ReturnT.SUCCESS;
    }

    private void birthdayRemind() {

        String time = LocalDateTime.now().format(DateTimeFormatter.ofPattern("MM-dd"));
        // -- 获取当天月日 生日的人员列表
        List<AdminUserDO> list = adminUserMapper.selectList(new LambdaQueryWrapper<AdminUserDO>()
                .like(AdminUserDO::getBirthdayDay, time));
        if (!list.isEmpty()) {

            //获取参数
            XxlJobContext xxlJobContext = XxlJobContext.getXxlJobContext();
            String jobParam = xxlJobContext.getJobParam();
            if (!StringUtil.isNullOrEmpty(jobParam)) {

                // 解析参数
                JSONObject object = new JSONObject(jobParam);
                Long roleId = object.getLong("roleId");

                Set<Long> userIds = permissionService.getUserRoleIdListByRoleId(CollectionUtils.singleton(roleId));
                //获得用户组中 用户信息
                List<AdminUserDO> userDOs = userService.getUserList(userIds);
                for (AdminUserDO adminUserDO : userDOs) {
                    if( adminUserDO.getOpenId() != null && !adminUserDO.getOpenId().isEmpty()) {

                        subscribeMessageSendApi.sendCompanyNotice(NoticeConvert.INSTANCE.convertBirthday(
                                adminUserDO.getOpenId(), "formal"));
                    }
                }

                log.info("生日提醒，生日为{}，人员为{}", time, convertList(list, AdminUserDO::getNickname));
            }
        }
    }
}
